import ddf.minim.*;
import ddf.minim.ugens.*;

String[] sound_files = {
  "BD.aif", "SD.aif", "HH.aif", "OH.aif", "HC.aif"
};

int beats = 16;
int dim = 40;
int margin = 2;
int bpm = 120;
int beat = 0;
Drum[] drums;

Minim minim;
AudioOutput out;

class Drum
{
  Sampler sound;
  Multiplier vol;
  int[] data;

  Drum( Sampler samp, AudioOutput out )
  {
    sound = samp;
    vol = new Multiplier( 1.0 );
    samp.patch( vol ).patch( out );
    data = new int[ beats ];
    for ( int idx = 0; idx < data.length; ++idx ) {
      data[idx] = 0;
    }
  }

  void tick( int beat )
  {
    int what = data[beat];
    if ( what == 2 ) {
      vol.setValue( 1.0 );
    } else if ( what == 1 ) {
      vol.setValue( 0.5 );
    } else {
      return;
    }
    sound.trigger();
  }
}

class Tick implements Instrument
{
  void noteOn( float dur )
  {
    for ( int idx = 0; idx < drums.length; ++idx ) {
      drums[idx].tick( beat );
    }
  }

  void noteOff()
  {
    beat = (beat + 1) % beats;
    out.setTempo( bpm );
    out.playNote( 0.0, 0.25, this );
  }
}

void setup()
{
  size( dim * beats + 1, dim * sound_files.length + 1 );
  stroke( 0 );

  minim = new Minim( this );
  out = minim.getLineOut();
  drums = new Drum[sound_files.length];

  for ( int idx = 0; idx < sound_files.length; ++idx ) {
    String fname = sound_files[idx];
    Sampler sound = new Sampler( fname, 4, minim );
    drums[idx] = new Drum( sound, out );
  }

  out.setTempo( bpm );
  out.playNote( 0, 0.25, new Tick() );
}

void draw()
{
  background( 200 );
  for ( int idx = 0; idx < drums.length; ++idx ) {
    Drum d = drums[idx];
    for ( int b = 0; b < beats; ++b ) {
      int what = d.data[b];
      if ( what == 2 ) {
        fill( 255, 180, 180 );
      } else if ( what == 1 ) {
        fill( 250, 250, 0 );
      } else {
        fill( 100, 100, 100 );
      }
      rect( dim * b + margin, dim * idx + margin, 
        dim - 2*margin, dim - 2*margin );
    }
  }

  noFill();
  rect( dim * beat, 0, dim, dim * sound_files.length );
}

void mousePressed()
{
  int dn = int( mouseY / dim );
  int i = int( mouseX / dim );
  drums[dn].data[i] = (drums[dn].data[i]+1);
}
